/*
 * Broadcom Proprietary and Confidential. Copyright 2016 Broadcom
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior
 * written permission of Broadcom Corporation.
 */

/** @file
 *
 * Bluetooth Classic RFCOMM and BLE GATT Dual mode sample application
 *
 * Features demonstrated
 *  - WICED BT RFCOMM server APIs
 *
 * On startup this demo:
 *  - Initializes the wiced_bt subsystem
 *  - Starts the GATT and RFCOMM server
 *  - Waits for clients to connect
 *
 * Application Instructions
 *   Connect a PC terminal to the serial port of the WICED Eval board,
 *   then build and download the application as described in the WICED
 *   Quick Start Guide
 *
 */
#include "wiced_bt_cfg.h"
#include <string.h>
#include <stdio.h>
#include "wiced.h"
#include "wiced_bt_dev.h"
#include "wiced_bt_ble.h"
#include "wiced_bt_gatt.h"
#include "wiced_bt_cfg.h"
#include "wiced_bt_gatt_db.h"
#include "bt_rfcomm_server.h"
#include "wiced_bt_stack.h"



/******************************************************
 *                      Macros
 ******************************************************/

/******************************************************
 *                    Constants
 ******************************************************/
 static wiced_bt_device_address_t hWbda={0x30,0x14,0x09,0x01,0x15,0x53};//{0x43,0x34,0x1B,0x00,0x1F,0xAC};

/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Function Declarations
 ******************************************************/
static wiced_bt_dev_status_t bt_dualmode_server_management_cback( wiced_bt_management_evt_t event, wiced_bt_management_evt_data_t *p_event_data );
static void bt_dualmode_server_init( void );
extern void ble_proximity_reporter_init( void );
extern void bt_rfcomm_server_init( void );

/******************************************************
 *               Variable Definitions
 ******************************************************/
uint8_t bluetooth_device_name[249] = WICED_BLUETOOTH_DEVICE_NAME;

void inq_cb (wiced_bt_dev_inquiry_scan_result_t *p_inq, uint8_t *p)
{

	wiced_result_t rst;

	uint8_t *p_eir_data;
	UINT8 remote_name_len=p[0];
	uint8_t name[20];
	UINT8 i;

	WPRINT_BT_APP_INFO(("\nBluetooth Address: [%02X:%02X:%02X:%02X:%02X:%02X] ",
		p_inq->remote_bd_addr[0], p_inq->remote_bd_addr[1], p_inq->remote_bd_addr[2], p_inq->remote_bd_addr[3],
		p_inq->remote_bd_addr[4], p_inq->remote_bd_addr[5]));
	WPRINT_BT_APP_INFO(("device class: 0x%02x%02x%02x\n", p_inq->dev_class[0], p_inq->dev_class[1],p_inq->dev_class[2]));

	if(0x1f == p_inq->dev_class[1]){//To pair with the other bcm43341.
		wiced_bt_cancel_inquiry();
		memcpy(hWbda, p_inq->remote_bd_addr, 6);

		rst = wiced_bt_dev_sec_bond(p_inq->remote_bd_addr, 0, BT_TRANSPORT_BR_EDR, 0, NULL);
		WPRINT_BT_APP_INFO( ("wiced_bt_dev_sec_bond rst %d\n", rst));

	}
	else if(0x5a == p_inq->dev_class[0]){ //To pair with my phone.
		wiced_bt_cancel_inquiry();
		memcpy(hWbda, p_inq->remote_bd_addr, 6);

		rst = wiced_bt_dev_sec_bond(p_inq->remote_bd_addr, 0, BT_TRANSPORT_BR_EDR, 0, NULL);
		WPRINT_BT_APP_INFO( ("wiced_bt_dev_sec_bond rst %d\n", rst));

	}

}

/******************************************************
 *               Function Definitions
 ******************************************************/
void application_start( )
{
    /* Initialize Bluetooth controller and host stack */
#ifdef WICED_DCT_INCLUDE_BT_CONFIG
    {
        /* Configure the Device Name and Class of Device from the DCT */
         platform_dct_bt_config_t* dct_bt_config;
        wiced_dct_read_lock( (void**) &dct_bt_config, WICED_TRUE, DCT_BT_CONFIG_SECTION, 0, sizeof(platform_dct_bt_config_t) );
        WPRINT_APP_INFO( ("DCT BT NAME: %s \r\n", dct_bt_config->bluetooth_device_name) );
		WPRINT_APP_INFO( ( "Local Bluetooth Address: [%02X:%02X:%02X:%02X:%02X:%02X]\n",
			dct_bt_config->bluetooth_device_address[0], dct_bt_config->bluetooth_device_address[1], 
			dct_bt_config->bluetooth_device_address[2], dct_bt_config->bluetooth_device_address[3], 
			dct_bt_config->bluetooth_device_address[4], dct_bt_config->bluetooth_device_address[5]) );

		
        strlcpy((char*)bluetooth_device_name, (char*)dct_bt_config->bluetooth_device_name, sizeof(bluetooth_device_name));
        wiced_bt_cfg_settings.device_name = bluetooth_device_name;
        WPRINT_APP_INFO( ("WICED DCT BT DEVICE CLASS : %02x %02x %02x\r\n", dct_bt_config->bluetooth_device_class[0],
                dct_bt_config->bluetooth_device_class[1],dct_bt_config->bluetooth_device_class[2]) );
        memcpy(wiced_bt_cfg_settings.device_class, dct_bt_config->bluetooth_device_class, sizeof(dct_bt_config->bluetooth_device_class));
        wiced_dct_read_unlock( (void*) dct_bt_config, WICED_TRUE );
    }
#endif

    wiced_bt_stack_init( bt_dualmode_server_management_cback, &wiced_bt_cfg_settings, wiced_bt_cfg_buf_pools );
}

void bt_dualmode_server_init( void )
{
    //ble_proximity_reporter_init();
    bt_rfcomm_server_init();
}

/* Bluetooth management event handler */
static wiced_bt_dev_status_t bt_dualmode_server_management_cback( wiced_bt_management_evt_t event, wiced_bt_management_evt_data_t *p_event_data )
{
    wiced_bt_device_address_t bda;
	wiced_bt_dev_inq_parms_t inq;
	wiced_result_t rst;
	int i;
	uint8_t write_bd[7];


	inq.mode =BTM_GENERAL_INQUIRY;
	inq.duration =30;
	inq.filter_cond_type=0;

    WPRINT_APP_INFO( ( "Bluetooth Management Event: 0x%x\n", event ) );

    wiced_bt_dev_status_t status = WICED_BT_SUCCESS;
    switch ( event )
    {
    case BTM_ENABLED_EVT:
        /* Bluetooth controller and host stack enabled */
        WPRINT_APP_INFO( ( "Bluetooth enabled (%s)\n", ( (p_event_data->enabled.status == WICED_BT_SUCCESS) ? "success":"failure" ) ) );

        if ( p_event_data->enabled.status == WICED_BT_SUCCESS )
        {
            wiced_bt_dev_read_local_addr( bda );
            WPRINT_APP_INFO( ( "Local Bluetooth Address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", bda[0], bda[1], bda[2], bda[3], bda[4], bda[5] ) );

#ifdef WICED_DCT_INCLUDE_BT_CONFIG
    {
        /* Configure the Device Address from the DCT */
         platform_dct_bt_config_t* dct_bt_config;
        wiced_dct_read_lock( (void**) &dct_bt_config, WICED_TRUE, DCT_BT_CONFIG_SECTION, 0, sizeof(platform_dct_bt_config_t) );
        WPRINT_APP_INFO( ("WICED DCT BT ADDR [0x%02x:0x%02x:0x%02x:0x%02x:0x%02x:0x%02x]\r\n",
                dct_bt_config->bluetooth_device_address[0], dct_bt_config->bluetooth_device_address[1],
                dct_bt_config->bluetooth_device_address[2], dct_bt_config->bluetooth_device_address[3],
                dct_bt_config->bluetooth_device_address[4], dct_bt_config->bluetooth_device_address[5]) );
        wiced_bt_set_local_bdaddr ( dct_bt_config->bluetooth_device_address );
	    wiced_dct_read_unlock( (void*) dct_bt_config, WICED_TRUE );
    }
#endif
		wiced_bt_dev_read_local_addr( bda );
		WPRINT_APP_INFO( ( "Local Bluetooth Address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", bda[0], bda[1], bda[2], bda[3], bda[4], bda[5] ) );

    /* Enable proximity reporter and rfcomm server */
        bt_dualmode_server_init();
		//2016wiced_bt_start_inquiry(&inq,inq_cb);

        }
        break;
	case BTM_PAIRING_IO_CAPABILITIES_BR_EDR_RESPONSE_EVT:
			WPRINT_BT_APP_INFO(("IO_CAPABILITIES_BR_EDR_RESPONSE_EVT auth, %d io cap%d oob%d\n",
						p_event_data->pairing_io_capabilities_br_edr_response.auth_req,
						p_event_data->pairing_io_capabilities_br_edr_response.io_cap,
						p_event_data->pairing_io_capabilities_br_edr_response.oob_data));
			break;
	case BTM_PAIRING_IO_CAPABILITIES_BLE_REQUEST_EVT:
			WPRINT_BT_APP_INFO(("IO_CAPABILITIES_BR_EDR_RESPONSE_EVT auth, %d init_keys%d io cap%d oob%d\n",
						p_event_data->pairing_io_capabilities_ble_request.auth_req,
						p_event_data->pairing_io_capabilities_ble_request.init_keys,
						p_event_data->pairing_io_capabilities_ble_request.local_io_cap,
						p_event_data->pairing_io_capabilities_ble_request.init_keys));
			break;
	case BTM_PAIRING_IO_CAPABILITIES_BR_EDR_REQUEST_EVT://8
			WPRINT_BT_APP_INFO(("BTM_PAIRING_IO_CAPABILITIES_BR_EDR_REQUEST_EVT %d io cap%d oob%d\n",
						p_event_data->pairing_io_capabilities_br_edr_request.auth_req,
						p_event_data->pairing_io_capabilities_br_edr_request.local_io_cap,
						p_event_data->pairing_io_capabilities_br_edr_request.oob_data));
			if(p_event_data->pairing_io_capabilities_br_edr_request.is_orig)
				WPRINT_BT_APP_INFO(("\tI'm the Initiator!\n"));
			
			p_event_data->pairing_io_capabilities_br_edr_request.local_io_cap = 
				BTM_IO_CAPABILITIES_DISPLAY_AND_YES_NO_INPUT;//BTM_IO_CAPABILITIES_DISPLAY_ONLY;
			p_event_data->pairing_io_capabilities_br_edr_request.auth_req = BTM_AUTH_ALL_PROFILES_YES;

			break;


	case BTM_PIN_REQUEST_EVT://  3
		WPRINT_BT_APP_INFO( ( "BTM_PIN_REQUEST_EVT %s\n", p_event_data->pin_request.bd_name));
	    wiced_bt_dev_pin_code_reply (p_event_data->pin_request.bd_addr, WICED_BT_SUCCESS, 4,"0000");
		break;

    case BTM_LOCAL_IDENTITY_KEYS_UPDATE_EVT:
		WPRINT_BT_APP_INFO(("\tBTM_LOCAL_IDENTITY_KEYS_UPDATE_EVT!\n"));
		break;

    case BTM_USER_CONFIRMATION_REQUEST_EVT:
		WPRINT_BT_APP_INFO(("BTM_USER_CONFIRMATION_REQUEST_EVT %d %d %d\n",
			p_event_data->user_confirmation_request.numeric_value, 
			p_event_data->user_confirmation_request.local_authentication_requirements,
			p_event_data->user_confirmation_request.remote_authentication_requirements));
        /* User confirmation request for pairing (sample app always accepts) */
        wiced_bt_dev_confirm_req_reply ( WICED_BT_SUCCESS, p_event_data->user_confirmation_request.bd_addr );
        break;


    case BTM_PAIRING_COMPLETE_EVT:
        /* Pairing complete */
		if(p_event_data->pairing_complete.pairing_complete_info.br_edr.status==WICED_BT_SUCCESS){
			WPRINT_BT_APP_INFO( ( "BTM_PAIRING_COMPLETE_EVT: success! %d\n", 
				p_event_data->pairing_complete.bonding_status));
			//bt_rfcomm_server_init2(p_event_data->pairing_complete.bd_addr);
		}
		else{
			WPRINT_BT_APP_INFO( ( "BTM_PAIRING_COMPLETE_EVT: failed! status %d bonding_status%d\n",
				p_event_data->pairing_complete.pairing_complete_info.br_edr.status,
				p_event_data->pairing_complete.bonding_status));
		}
		break;



    case BTM_BLE_ADVERT_STATE_CHANGED_EVT:
        /* adv state change callback */
        WPRINT_BT_APP_INFO( ( "---->>> New ADV state: %d\n", p_event_data->ble_advert_state_changed ) );
        break;

    default:
        WPRINT_BT_APP_INFO( ( "Unhandled Bluetooth Management Event: 0x%x\n", event ) );
        break;
    }

    return ( status );
}
